/*
 * 3pi-linefollower-pid - demo code for the Pololu 3pi Robot
 * 
 * This code will follow a black line on a white background, using a
 * PID-based algorithm.
 *
 * http://www.pololu.com/docs/0J21
 * http://www.pololu.com
 * http://forum.pololu.com
 *
 */

// The 3pi include file must be at the beginning of any program that
// uses the Pololu AVR library and 3pi.
#include <pololu/3pi.h>

// This include file allows data to be stored in program space.  The
// ATmega168 has 16k of program space compared to 1k of RAM, so large
// pieces of static data should be stored in program space.
#include <avr/pgmspace.h>
#include "3pi-demo-program.h"
#include "SensorTest.h"

// A couple of simple tunes, stored in program space.
const char go[] PROGMEM = "L16 cdegreg4";
const char beep_1[] PROGMEM = "!e32";

// This is the line_follow function using a PID algorithm
void line_follow()
{
	unsigned int sensors[5]; // an array to hold sensor values
	unsigned int last_proportional=0;
	unsigned int position;	// stores the sensor position output
	const int max = 60;
	int power_difference = 0;
	long integral=0;
	static long current_time = 0;
	static long elapsed_time = 0;
	static long last_read = 0;
	static char seconds = 0;

	// Auto-calibration of the sensors.
	calibrate("Line", "Follower");

	// Display calibrated values as a bar graph.
	while(!button_is_pressed(BUTTON_B))
	{
		// Read the sensor values and get the position measurement.
		unsigned int position = read_line(sensors,IR_EMITTERS_ON);

		// Display the position measurement, which will go from 0
		// (when the leftmost sensor is over the line) to 4000 (when
		// the rightmost sensor is over the line) on the 3pi, along
		// with a bar graph of the sensor readings.  This allows you
		// to make sure the robot is ready to go.
		clear();
		print_long(position);
		lcd_goto_xy(0,1);
		display_readings(sensors);
		delay_ms(50);
	}
	wait_for_button_release(BUTTON_B);

	clear();
	print("Go!");		

	// This is the "line following loop" - it will run until button B is pressed
	while(!button_is_pressed(BUTTON_B))
	{
		// Get the position of the line.  Note that we *must* provide
		// the "sensors" argument to read_line() here, even though we
		// are not interested in the individual sensor readings.
		position = read_line(sensors,IR_EMITTERS_ON);

		current_time = get_ms();
		elapsed_time += current_time - last_read;
		last_read = current_time;
		seconds = ((elapsed_time/1000)%60)%10;
		// print position and sensor bar every 2 seconds
		if (seconds >= 2)
		{
			elapsed_time = 0;	
			clear();
//			print_long(position);
			print_long(power_difference);
			lcd_goto_xy(0,1);
			display_readings(sensors);
			play_from_program_space(beep_1);
		}

		// The "proportional" term should be 0 when we are on the line.
		int proportional = ((int)position) - 2000;

		// Compute the derivative (change) and integral (sum) of the
		// position.
		int derivative = proportional - last_proportional;
		integral += proportional;

		// Remember the last position.
		last_proportional = proportional;

		// Compute the difference between the two motor power settings,
		// m1 - m2.  If this is a positive number the robot will turn
		// to the right.  If it is a negative number, the robot will
		// turn to the left, and the magnitude of the number determines
		// the sharpness of the turn.
//		int power_difference = proportional/20 + integral/10000 + derivative*3/2;
		power_difference = proportional/10 + integral/10000 + derivative*3/2;

		// Compute the actual motor settings.  We never set either motor
		// to a negative value.
		if(power_difference > max)
			power_difference = max;
		if(power_difference < -max)
			power_difference = -max;

		if(power_difference < 0)
			set_motors(max+power_difference, max);
		else
			set_motors(max, max-power_difference);
	}
	return;
}

// Local Variables: **
// mode: C **
// c-basic-offset: 4 **
// tab-width: 4 **
// indent-tabs-mode: t **
// end: **
